unit GenUtils;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Types, Registry, ShellAPI;


type
  TDWORDID = array[0..3] of Char;

function IntIndex(I: Integer; const A: array of Integer): Integer;
function HighSetBit(I: Cardinal): Integer;

type
  TRange = record
    Min, Max: Extended;
  end;

function EnsureRange(const AValue: Extended; const ARange: TRange): Extended; overload;

function RandPermut(n: Integer; r: Integer = -1): TIntegerDynArray;

const
  _2Pi = 2 * Pi;

function dBToLin(const dB: Extended): Extended;
function LinTodB(const Lin: Extended): Extended;
function MiddBLevel(const L: array of Extended): Extended;

function RoundToLen(const AValue: Double; ADigits: Integer): Double;
function FracLen(const AValue: Extended; ADigits: Integer): Integer;

resourcestring
  SDays = 'd';

function TimeToStrEx(const Time: TDateTime): string;

function MessageBoxResFmt(Msg: PResStringRec; const Args: array of const;
  Flags: Integer): Integer;
procedure SetEnableControls(const AControls: array of TControl; AEnable: Boolean);
procedure PlaceControl(AControl: TControl; const APoint: TPoint);
procedure ChangeWindowStyle(Handle: HWND; Incl, Excl: DWORD);
procedure SetTopmost(Handle: HWND; IsTopmost: Boolean);

function EnsureFileExt(const FileName, Extension: string): string;

type
  TDropFiles = class(TStrings)
  private
    Handle: HDROP;
  protected
    function Get(Index: Integer): string; override;
    function GetCount: Integer; override;
    function GetPoint: TPoint;
  public
    constructor Create(Message: TWMDropFiles);
    destructor Destroy; override;
    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    procedure Insert(Index: Integer; const S: string); override;
    property Point: TPoint read GetPoint;
  end;


const
  SWindowLeft       = 'WindowLeft';
  SWindowTop        = 'WindowTop';
  
var
  CompanyName, ProductName: string;

function RegKey: string;


implementation

uses Math, Forms;


function IntIndex(I: Integer; const A: array of Integer): Integer;
begin
  for Result := 0 to High(A) do
    if A[Result] = I then
      Exit;
  Result := -1;
end;

function HighSetBit(I: Cardinal): Integer;
asm
        BSR     EAX,EAX
end;


function EnsureRange(const AValue: Extended; const ARange: TRange): Extended;
begin
  Result := AValue;
  with ARange do
  begin
    Assert(Min <= Max);
    if Result < Min then
      Result := Min;
    if Result > Max then
      Result := Max;
  end;
end;


function RandPermut(n: Integer; r: Integer = -1): TIntegerDynArray;
var
  I, J, SubIndex, Number: Integer;
begin
  if (r < 0) or (r > n) then
    r := n;
  with TBits.Create do
    try
      Size := n;
      SetLength(Result, r);
      for I := 0 to r - 1 do
      begin
        SubIndex := Random(n - I);
        J := 0;
        Number := 0;
        while True do
        begin
          while Bits[Number] do
            Inc(Number);
          if J = SubIndex then
            Break;
          Inc(Number);
          Inc(J);
        end;
        Result[I] := Number;
        Bits[Number] := True;
      end;
    finally
      Free;
    end;
end;


function dBToLin(const dB: Extended): Extended;
begin
  Result := Power(2, dB / 6);
end;

function LinTodB(const Lin: Extended): Extended;
begin
  Result := 6 * Log2(Lin);
end;

function MiddBLevel(const L: array of Extended): Extended;
var
  Sum: Extended;
  I: Integer;
begin
  Sum := 0;
  for I := 0 to High(L) do
    Sum := Sum + dBToLin(L[I]);
  Result := LinTodB(Sum / Length(L));
end;


function RoundToLen(const AValue: Double; ADigits: Integer): Double;
begin
  Result := RoundTo(AValue, -FracLen(AValue, ADigits));
end;

function FracLen(const AValue: Extended; ADigits: Integer): Integer;
begin
  if AValue <> 0.0 then
    Result := ADigits - Trunc(Log10(Abs(AValue))) - 1
  else
    Result := 0;
end;


function TimeToStrEx(const Time: TDateTime): string;
begin
  DateTimeToString(Result, 'HH:mm:ss', Time);
  if Time >= 1 then
    Result := Format('%d%s %s', [Trunc(Time), SDays, Result]);
end;


function MessageBoxResFmt(Msg: PResStringRec; const Args: array of const;
  Flags: Integer): Integer;
begin
  Result := Application.MessageBox(PChar(Format(LoadResString(Msg), Args)),
    PChar(Application.Title), Flags);
end;

procedure SetEnableControls(const AControls: array of TControl; AEnable: Boolean);
var
  i: Integer;
begin
  for i := 0 to High(AControls) do
    AControls[i].Enabled := AEnable;
end;

procedure PlaceControl(AControl: TControl; const APoint: TPoint);
begin
  with AControl, APoint do
    SetBounds(X - Width div 2, Y - Height div 2, Width, Height);
end;

procedure ChangeWindowStyle(Handle: HWND; Incl, Excl: DWORD);
var
  AStyle: DWORD;
begin
  AStyle := GetWindowLong(Handle, GWL_STYLE);
  if AStyle <> 0 then
    SetWindowLong(Handle, GWL_STYLE, (AStyle or Incl) and not Excl);
end;

procedure SetTopmost(Handle: HWND; IsTopmost: Boolean);
const
  Topmost: array[Boolean] of Cardinal = (HWND_NOTOPMOST, HWND_TOPMOST);
begin
  SetWindowPos(Handle, Topmost[IsTopmost], 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE
    or SWP_NOACTIVATE);
end;

function EnsureFileExt(const FileName, Extension: string): string;
begin
  Result := FileName;
  if not SameText(ExtractFileExt(Result), Extension) then
    Result := Result + Extension;
end;

{ TDropFiles }

procedure TDropFiles.Clear;
begin
end;

constructor TDropFiles.Create(Message: TWMDropFiles);
begin
  Handle := Message.Drop;
end;

procedure TDropFiles.Delete(Index: Integer);
begin
end;

destructor TDropFiles.Destroy;
begin
  DragFinish(Handle);
  inherited Destroy;
end;

function TDropFiles.Get(Index: Integer): string;
begin
  SetLength(Result, DragQueryFile(Handle, Index, nil, 0));
  DragQueryFile(Handle, Index, Pointer(Result), Length(Result) + 1);
end;

function TDropFiles.GetCount: Integer;
begin
  Result := DragQueryFile(Handle, $FFFFFFFF, nil, 0);
end;

function TDropFiles.GetPoint: TPoint;
begin
  DragQueryPoint(Handle, Result);
end;

procedure TDropFiles.Insert(Index: Integer; const S: string);
begin
end;


function RegKey: string;
begin
  Result := '\Software\' + CompanyName + '\' + ProductName;
end;

initialization
  CompanyName := 'eCat';
  ProductName := 'MyProduct';

end.
